# -*- coding: utf-8 -*-
###########################################################################
# 
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
# eole@ac-dijon.fr 
#  
###########################################################################
"""aggrégateur de fichiers access.log (du jour et zippés)"""
import zip
import logparser
import datetime
import os
from os import path as osp
from errors import DataError
from config import LOGFILE_PATH, ZIPPED_LOGFILE_PATH, LOGFILE_NAME
from tools import today_day, get_day_str, compare_func
import re

def available_days_in_logs(logfile_name=LOGFILE_NAME, logfile_dir=LOGFILE_PATH, zipped_logfile_dir=ZIPPED_LOGFILE_PATH, force=None):
    """déterminons demandé le jour en fonction du nom de fichier
    classement des fichiers de log 'access.log.1.gz', 'access.log.2.gz', 'access.log.3.gz', 
    en fonction du jour de la semaine
    return {'week':['jour', 'nom du fichier']}
    """
    # fichiers correspondants à la semaine et à la semaine précédente
    logfiles = {'week':[], 'lastweek':[]}
    files = list(collect_log_filenames(zipped_logfile_dir))
    # ajoute le fichier de log non compressé
    # dans le cas ou les zip et le fichier de log sont dans des rep differents
    if logfile_name not in files: files.append(logfile_name)
    # tri en fonction du logrotate ('access.log.1.gz', 'access.log.2.gz', 'access.log.3.gz', ...)
    files.sort(compare_func)
    # jour courant (celui correspondant au fichier access.log non compressé)
    # possiblité de forcer ce jour (ce positionner dans le passé ou le présent)
    if not force: today = today_day()
    else: today = force
    # algorithme de classement en semaine, semaine dernière
    for index, fname in enumerate(files):
        relative_indice = today - index
        if relative_indice >= -7:
            indice = (relative_indice)%7
            day_name = get_day_str(indice)
            if relative_indice >= 0:
                logfiles['week'].append((day_name, fname))
            else:
                logfiles['lastweek'].append((day_name, fname))
    return logfiles                
    
def collect_log_filenames(path, max=15):
    """liste les noms de fichiers de log (compressés ou non) dans un répertoire donné"""
    if osp.isdir(path):
        for fname in os.listdir(path):
            if LOGFILE_NAME in fname:
                yield fname
    else:
        raise DataError, "repertoire non valide"


