# -*- coding: utf-8 -*-
###########################################################################
# 
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
# eole@ac-dijon.fr 
#  
###########################################################################

"""Module de stockage et de manipulation structurée des données de parsing des log """

import os.path , csv
from errors import DataError
from tools import isoformat_to_datetime
from zip import get_handle
from os.path import splitext
import datetime
import commands

class CSVStorage:
    """gestion des données issues d'un fichier de type csv
    (exemple : acces.log)"""
    
    def __init__(self, filename=None, keywords=[]):
        """
        filename: chemin du fichier de sauvegarde
        """
        if filename:
            self.filename = filename
            self.storage_file = os.path.abspath(os.path.normpath(self.filename))
        if not keywords:
            raise Exception, "il faut renseigner les noms des champs"
        self.keywords = keywords
        # contenu de self.data filtré par la méthode self.filter
        self.filtered = []

    ## chargement et lecture des données

    def load_filter_data(self, expression=None):
        """
        recuperation des logs contenant expression
        :expression: champ recherché
        """
        error = "aucun surf present sur cette periode"
        # vidons les données
        self.data = []
        # le fichier doit exister
        if not self.storage_file:
            raise DataError, error
        try:
            if splitext(self.filename)[1] == '.gz':
#                fhandle = get_handle(self.storage_file)
                cmd = "zgrep %s %s"%(expression, self.storage_file)
            else:
#                fhandle = file(self.storage_file, 'r')
                cmd = 'grep %s %s'%(expression, self.storage_file)
            result = commands.getoutput(cmd)
            self.data = result.splitlines()
#            self.data = fhandle.readlines()
#            csv_reader = csv.reader(fhandle)
#            for row in csv_reader:
#                kwargs = dict(zip(self.keywords, row))
#                self.data.append(kwargs)
        except:
            raise DataError, error

    def construct_date(self, hour):
        """ recupere la date du fichier de log
            renvoit quelque chose comme "2007:06:12 hour:
        """
        first_file_line = commands.getoutput('grep -m1 , %s'%self.storage_file)
        date = first_file_line.split(',')[0].split()[0]
        return "%s %s:"%(date, hour)

    def zconstruct_date(self, hour):
        """ recupere la date du fichier de log depuis un .gz
            renvoit quelque chose comme "2007:06:12 hour:"""
        first_file_line = commands.getoutput('zgrep -m1 , %s'%self.storage_file)
        date = first_file_line.split(',')[0].split()[0]
        return "%s %s:"%(date, hour)

    def load_schedule_data(self, begin, end):
        """ recupere les logs entre begin et end """
        error = "aucun surf sur cette periode"
        # vidons les données
        self.data = []
        # le fichier doit exister
        if not self.storage_file:
            raise DataError, error
        try:
            if splitext(self.filename)[1] == '.gz':
                if begin == 0 and end == 23:
                    cmd = "zcat %s"%self.storage_file
                    result = commands.getoutput(cmd)
                else:
                    dates = []
                    for i in range(begin, end):
                        dates.append(self.zconstruct_date(i))
                    result = ""
                    for expression in dates:
                        cmd = "zgrep '%s' %s"%(expression, self.storage_file)
                        result += commands.getoutput(cmd)
            else:
                if begin == 0 and end == 23:
                    cmd = "cat %s"%self.storage_file
                    result = commands.getoutput(cmd)
                else:
                    dates = []
                    for i in range(begin, end):
                        dates.append(self.construct_date(i))
                    result = ""
                    for expression in dates:
                        cmd = "grep '%s' %s"%(expression, self.storage_file)
                        result += commands.getoutput(cmd)
            self.data = result.splitlines()
        except:
            raise DataError, error
        
    def get_data(self):
        """Récupère les données à enregistrer
        """
        return self.data

    def filter_schedule(self, begin, end, keyword='date'):
        """filtre un contenu suivant une tranche horaire
        begin : datetime debut
        end : datetime fin
        """
        if end < begin:
            raise DataError, "le champ de début de date est supérieur au champ de fin de date"
        lines = self.data
        csv_reader = csv.reader(lines)
        toto = []
        for row in csv_reader:
            kwargs = dict(zip(self.keywords, row))
            toto.append(kwargs)
        return toto
    
    def get_user_lines(self, user):
        """ renvoie les lignes des logs de username """
        for data in self.data:
            if user in data.split(',')[1]:
                yield data

    def filter_user(self, user):
        """ filtre un contenu selon un nom de user"""
        if not user:
            raise DataError, "Le nom d'utilisateur est vide"
        lines = list(self.get_user_lines(user))
        csv_reader = csv.reader(lines)
        toto = []
        for row in csv_reader:
            kwargs = dict(zip(self.keywords, row))
            toto.append(kwargs)
        return toto
        
    def get_ip_lines(self, ip):
        """ renvoie les lignes des logs de ipname """
        for data in self.data:
            if ip in data.split(',')[2]:
                yield data

    def filter_ip(self, ip):
        """ filtre un contenu selon une ip
        """
        if not ip:
            raise DataError, "Le nom d'utilisateur est vide"
        lines = list(self.get_ip_lines(ip))
        csv_reader = csv.reader(lines)
        toto = []
        for row in csv_reader:
            kwargs = dict(zip(self.keywords, row))
            toto.append(kwargs)
        return toto
    
    def get_denied_lines(self, keyword):
        """ renvoie les lignes des logs de ipname """
        for data in self.data:
            if keyword in data:
                yield data

    def filter_denied(self, keyword):
        """ filtre un contenu selon une ip
        """
        if not keyword:
            raise DataError, "Le nom d'utilisateur est vide"
        lines = list(self.get_denied_lines(keyword))
        csv_reader = csv.reader(lines)
        denied = []
        for row in csv_reader:
            kwargs = dict(zip(self.keywords, row))
            denied.append(kwargs)
        return denied
    
    def set_data(self, data):
        """Renseigne les données à enregistrer
        *attention*, remplace **toutes** les données et ecrase
        le self.data existant. Il faut que les données soient
        de même forme que ce qui est renseigné dans self.keywords
        """
        self.data = data

def csv_storage_factory(filename, keywords, expression=''):
    """usine à csvstorage
    """
    c = CSVStorage(keywords=keywords, filename=filename) 
    c.load_filter_data(expression=expression)
    return c

def csv_schedule_storage_factory(filename, keywords, begin, end):
    """usine à csvstorage
    """
    c = CSVStorage(keywords=keywords, filename=filename) 
    c.load_schedule_data(begin, end)
    return c

def parselog(filename, keywords):
    """crée un parseur et renvoie les résultats    """
    c = csv_storage_factory(filename, keywords)
    return c.get_data()

def filter_denied(filename, keywords, exception_keyword='*DENIED*'):
    """crée un parseur et renvoie uniquement les urls filtrées 

    **filename** nom du fichier de log (acces.log)
    **keywords** champs, format du fichier de log
    """
    c = csv_storage_factory(filename, keywords, expression=exception_keyword)

    """
    **filter_keyword** nom du champ dans lequel se trouve l'exception
    **exception** filtres d'exceptions, phrase clef qui renseigne que dansguardian
    a repéré un site interdit
    """
    return c.filter_denied(exception_keyword)

def filter_schedule(keywords, begin, end, fname="access.log"):
    """lignes parsées par date
    keywords: liste des champs du fichier
    begin: datetime debut (format datetime)
    end : datetime fin
    fname: nom long du fichier de log
    keyword: nom du champ date 
    """
    c = csv_schedule_storage_factory(fname, keywords, begin=begin, end=end)
    return c.filter_schedule(begin, end)

def filter_user(keywords, user, fname="access.log"):
    """filtre les logs par username"""
    c = csv_storage_factory(fname, keywords, expression=user)
    return c.filter_user(user)

def filter_ip(keywords, ip, fname="access.log"):
    """ filtre les logs par Ip 
    """
    c = csv_storage_factory(fname, keywords, expression=ip)
    return c.filter_ip(ip)

