# -*- coding: utf-8 -*-
###########################################################################
# 
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
# eole@ac-dijon.fr 
#  
###########################################################################
"""génération de pages html"""

from tools import instance
from collect import available_days_in_logs
from logparser import parselog, filter_schedule, filter_user, filter_ip, filter_denied
from config import SCHEDULE, RESULT_PAGE, KEYWORDS, LOGFILE_PATH, LOGFILE_NAME
from os import path as osp

def weekform(context=None, schedule= SCHEDULE, result_page=RESULT_PAGE):
    """le formulaire de menu semaine et semaine dernière
    avec horaire
    context : un dictionnaire de menu
    """
    if not context:
        context = available_days_in_logs()
    context['schedule'] = schedule
    context['ip'] = ''
    context['user'] = ''
    context['result_page'] = result_page
    return instance("form.tmpl", context=context)

def ipform(context=None, result_page=RESULT_PAGE):
    """le formulaire de menu semaine et semaine dernière
    avec ip
    context : un dictionnaire de menu
    """
    if not context:
        context = available_days_in_logs()
    context['schedule'] = ''
    context['ip'] = True
    context['user'] = ''
    context['result_page'] = result_page
    return instance("form.tmpl", context=context)

def userform(context=None, result_page=RESULT_PAGE):
    """le formulaire de menu semaine et semaine dernière
    avec poste
    context : un dictionnaire de menu
    """
    if not context:
        context = available_days_in_logs()
    context['schedule'] = ''
    context['ip'] = '' 
    context['user'] = True
    context['result_page'] = result_page
    return instance("form.tmpl", context=context)

def deniedform(context=None, result_page='stat-refus.php'):
    """formulaire de génération de menu pour la page des sites refuses"""
    if not context:
        context = available_days_in_logs()
    context['result_page'] = result_page
    return instance("denied_form.tmpl", context=context)

def navigation(pgname=RESULT_PAGE, schedule=SCHEDULE, filename='access.log'):
    """generation de l'outil de navigation
    FIXME : la navigation sera incluse (une sortie en stream?)
    """
    horaires = []
    for horaire in schedule:
        horaires.append(horaire.split('-'))
    context = dict(schedule=horaires, pgname=pgname, filename=filename)
    return instance('navigation.tmpl', context=context)
        
def log_result(context=None, filename=None):
    """une page web de logs
    - pour une journée entière sur un fichier de logs entier
    """
    if not context:
        context = parselog(filename, keywords=KEYWORDS )
    return instance("log.tmpl", context=dict(context=context))

def log_schedule(begin, end, context=None, filename=osp.join(LOGFILE_PATH, LOGFILE_NAME), schedule=SCHEDULE):
    """une page web de logs
    - sur un fichier de logs donné
    - pour un intervalle de temps donné 
    """
    if not context:
        context = filter_schedule(keywords=KEYWORDS, begin=begin, end=end, fname=filename)
    
    # inclusion de la navigation
    horaires = []
    for horaire in schedule:
        horaires.append(horaire.split('-'))
    cont = dict(context=context)
    filename = osp.basename(filename)
    cont.update(dict(schedule=horaires, pgname=RESULT_PAGE, filename=filename))
    return instance("log.tmpl", context=cont)

def log_user(user, context=None, filename=osp.join(LOGFILE_PATH, LOGFILE_NAME)):
    """ une page web des logs utilisateurs
    """
    if not context:
        context = filter_user(keywords=KEYWORDS, user=user, fname=filename)
    filename = osp.basename(filename)
    cont = dict(context=context)
    return instance("user_log.tmpl", context=cont)

def log_ip(ip, context=None, filename=osp.join(LOGFILE_PATH, LOGFILE_NAME)):
    """ une page web des logs pour une ip
    """
    if not context:
        context = filter_ip(keywords=KEYWORDS, ip=ip, fname=filename)
    filename = osp.basename(filename)
    cont = dict(context=context)
    return instance("ip_log.tmpl", context=cont)

def log_denied(filename):
    """affichage des sites interdits """
    context = filter_denied(filename, keywords=KEYWORDS)
    return instance('denied.tmpl', context=dict(context=context))
