# -*- coding: utf-8 -*-
###########################################################################
# 
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
# eole@ac-dijon.fr 
#  
###########################################################################
"""utilitaires divers (formatage de dates, comparaison de fonctions)"""

import datetime
import time
import re
import os.path as osp

from Cheetah.Template import Template
from config import TEMPLATE_DIR


def timestamp_to_datetime(timestmp=time.time()):
    """objet datetime à partir d'une date unix (timestamp)"""
    return datetime.datetime.fromtimestamp(timestmp)

def today():
    """ou plus simplement datetime.today(), et aussi
    d.date() renvoie le jour courant du datetime d
    """
    return datetime.datetime.now().date()

def get_day_str(indice):
    """jour de la semaine en francais"""
    weekdays = ['lundi', 'mardi', 'mercredi','jeudi', 'vendredi', 'samedi', 'dimanche']
    return weekdays[indice]

def today_day():
    """indice du jour dans la semaine"""
    return today().weekday()

def isoformat_to_datetime(date, format='%Y.%m.%d %H:%M:%S'):
    """Le format propose decode une date et une heure au format iso:
    Année-Mois-Jour heures:minutes:secondes
    Les directives de formats dans ce cas sont:
      %Y = année sur 4 chiffres
      %m = mois sur 2 chiffres: 1..12
      %d = jour sur 2 chiffres: 1..31
      %H = heure sur 2 chiffres: 0..23
      %M = minutes sur 2 chiffres: 0..59
      %S = secondes sur 2 chiffres: 0..59
    """
    return datetime.datetime.fromtimestamp(
             time.mktime(
               time.strptime(date, format)))

def _get_nb_in_name(name):
    """nombre present dans un nom (de fichier),
    utilisé pour parser 'access.log.1.gz', 'access.log.2.gz', 'access.log.3.gz',
    """
    regexp = "[0-9]+"
    if re.search(regexp, name):
        name = re.findall(regexp, name)
        return int(name[0])
    else:
        return -1
    
def compare_func(a, b):
    """ fonction de comparaison """
    un = _get_nb_in_name(a)
    deux = _get_nb_in_name(b)
    return cmp(un, deux)


def instance(tplname, tpldir=TEMPLATE_DIR, context={}):
    """outil d'instance de template
    méthodologie de templating cheetah
    tplname: nom du template cheetah
    context: variables de template
    """
    t = Template(file=osp.join(tpldir, tplname), searchList=[context])
    return  str(t)
