<?
/*
=======================================================================

   filtrer les fichiers de logs
   $Id: stat-lib.inc,v 1.3 2004/05/05 09:42:41 gwen Exp $
=======================================================================
-
 Tout le parsing des logs est basé sur les timestamps (càd à partir de la date)

 Ce module permet de générer les listes déroulantes,
 générer et tester des dates unix,
 et proposer une sortie formatée (html) des données filtrées

- 
 Fonctions : 

 cetteSemaine 
 unixSemaine
 unixSemaineDerniere
 string FormCetteSemaine
 FormSemaineDerniere
 Intervalle
 IntervalleJour
 Parse
 ParseRefus
 FormatageResultat
 FormatageResultatRefus
 FormatageResultatIP
 FormatageResultatReferant
 FormatageResultatHost
 
 humanDate

 Serialize_result
 DeSerialize_result

 Remarques :

- séparation du parsing squid est squig_guard (les formats de fichiers sont différents)
- dissociation du traitement des listes déroulantes entre semaine et semaine dernière
  parce qu'il faut déterminer la liste des jours de cette semaine
- le tableau des jours de la semaine est une variable globale, ainsi que les durées unix
  standard (heure, jour...)


=======================================================================
*/

/*
=======================================================================

   génération des dates unix  pour la création des select boxes

=======================================================================
*/



  /*
    cetteSemaine
    -
    pour le formulaire sur cette semaine :
    génère un vecteur composé des jours de cette semaine (en ordre inversé)
    depuis dimanche dernier
    -
    Paramètres : 
    pas de paramètre
    -
    Résultat :
    vecteur des jours de cette semaine (en français)
    -
    Globales :
    $jours
    - 
  */

function cetteSemaine() {

  global $jours ;

  // ne pas travailler directement sur la variable globale...
  $vect = $jours ;

  // donne le nombre de jours d'ici à dimanche dernier
  $integer = strftime ("%w") ;
  if ( $integer == 0 ) {
  $integer = 6 ;
  }

  // update du vecteur jours   

  $i = 1 ;

  foreach ( $vect as $keys => $val ) 
    {

      if ( $i <= $integer ) 
	{

            $this_week[$keys]  = $val ;


	} else  
	  {

	  // ne fait rien

	  }
      $i++ ;
    }

  return $this_week ;
}

  /*
    unixSemaine
    -
    pour le traitement de cette semaine :
    génère un vecteur associatif jour_semaine => time_stamp
    afin de le réutiliser dans la génération de la liste déroulante select_semaine
    (les jours sont en ordre inversé)
    -
    Paramètres : 
    pas de paramètre
    -
    Résultat :
    vecteur des jours de cette semaine (en français) et des time_stamp de début de jour
    -
    Globales :
    $unix_jour
    cetteSemaine()
    - 
  */


function unixSemaine() 
{

  global $unix_jour ;

  $this_week = cetteSemaine() ;
  $maintenant = time() ;
  list( $debut_ce_jour , $fin_ce_jour )  = Intervalle( $maintenant ) ;
  
  $unix_date = $debut_ce_jour ;


  while ( $this_week )  {

    // commencer par vider le vecteur pour coordonner les dates avec les jours
    $val = array_pop($this_week) ;

    $unix_week[$val] = $unix_date ;
    $unix_date = $unix_date - $unix_jour ;        
    
  }

  return $unix_week ;
  
}


//=====================================================================


  /*
    unixSemaineDerniere
    -
    pour le traitement de la semaine dernière :
    génère un vecteur associatif jour_semaine_derniere => time_stamp
    afin de le réutiliser dans la génération de la liste déroulante select_semaine_dern

    -
    Paramètres : 
    pas de paramètre
    -
    Résultat
    vecteur des jours de la semaine dernière 
    (en français) et des time_stamp de début de jour
    -
    Références Globales :
    $unix_jour
    cetteSemaine()
    - 
  */

function unixSemaineDerniere() 
{

  global $jours, $unix_semaine, $unix_jour ;
  
  $semaine = abs(time() - $unix_semaine) ;

  $integer = strftime ("%w") + 6 ;
  $debut_sem_derniere  = mktime(0,0,0,date("m"),date("d")-$integer,date("Y"))  ;
  
 
  foreach ($jours as $val)  {
    
    $vect[$val] = $debut_sem_derniere ;
    $debut_sem_derniere = $debut_sem_derniere + $unix_jour ;
    
  }

  return $vect ;

}


/*
=======================================================================

   Création des Select Box pour cette semaine et 
   pour la semaine dernière 

=======================================================================
*/

  /*
    FormCetteSemaine
    -
    génére la liste déroulante html pour les jours de cette semaine
    -
    Paramètres : 
    pas de paramètre
    -
    Résultat :
    string html : une select box single choice
    -
    Globales :
    unixSemaine()
    - 
  */

function FormCetteSemaine() 
{
  
  $buffer = "<select name=\"str_select_week\">";
  
  $liste = cetteSemaine() ;
  $buffer .= "<option value=\"\">-------</option>" ;

  foreach ($liste as $keys => $val)  {
    $buffer .= "<option value=\"$keys\">$keys</option>" ;
  } 
  
  $buffer .= "</select>" ;
  
  
  return $buffer ;

}


//=====================================================================

  
  /*
    FormCetteSemaineDerniere
    -
    génére la liste déroulante html pour les jours de la semaine dernière 
    -
    Paramètres : 
    $semaine // vecteur des jours de la semaine courante 
    -
    Résultat :
    string html : une select box single choice
    -
    Globales :
    unixSemaine()
    - 

  */

function FormSemaineDerniere() 
{

  global $jours ;

  $liste = $jours ;

  $buffer = "<select name=\"str_select_lastweek\" size=\"1\" >";
  $buffer .= "<option value=\"\">-------</option>" ;

  foreach ( $liste as $keys => $val)
  {

    $buffer .= "<option value=\"$keys\">$keys</option>" ;

  } 
  
  $buffer .= "</select>" ;
    
  return $buffer ;

}


/*
=======================================================================

Gestion des intervalles de dates

=======================================================================
*/

  

  /*
    Intervalle
    -
    retourne les time_stamps début de jour et fin de jour pour une date donnée   
    -
    Paramètres : 
    un time_stamp
    -
    Résultat :
    array début du jour, fin du jour
    -
    Globales :
    $unix_jour
    - 
  */

function Intervalle( $time )
{
  
  global $unix_jour ;

  $date_debut_jour = mktime(0,0,0,date("m",$time),date("d",$time),date("Y",$time))  ;
  
  $date_fin_jour = $date_debut_jour + $unix_jour ;
  
  return array($date_debut_jour,$date_fin_jour) ;

}


  /*
    IntervalleJour
    -
    le test intervalle-jour consiste à
    savoir si une date apartient à une journée 
    déterminée par un intervalle de jour passé en paramètre
    -
    Paramètres : 
    time_stamp $time_log
    time_stamp $date_debut_jour
    time_stamp $date_fin_jour
    time_log est le time_stamp d'une ligne de log donnée
    debut et fin sont les extrémités de l'intervalle de la journée 
    -
    Résultat :
    0 ou 1 suivant que time_log appartient ou non au jour choisi
    -
    Globales :
    aucune
    - 
  */

function IntervalleJour($time_log,$date_debut_jour,$date_fin_jour) 
{

  // conversions pour la forme, on ne sait jamais time_log est une string...

	  $date_debut_jour = intval($date_debut_jour) ;
	  $date_fin_jour = intval($date_fin_jour) ;
	  $time_log = intval($time_log) ;


  if (($date_debut_jour <= $time_log) && ($time_log <= $date_fin_jour))  {
    
    return 1 ;
    
  } else  {
    
    return 0 ;
    
  }
  
}


/*
=======================================================================

Parseur et formatage des données stat

=======================================================================
*/



  /*
    Parse
    -
    parse les logs du data-access pour le proxy 
    -
    Paramètres : 
    time_stamp $date_debut_jour
    time_stamp $date_fin_jour
    int $i 
    debut et fin sont les extrémités de l'intervalle d'une journée 
    $i permet de déterminer quel fichier parser
    -
    Résultat :
    retourne le vecteur avec les données à présenter
    -
    Références Globales :
    $data_access_squid, $data_access_path, $data_access_squid_sem_dern ;
    - 
  */

function Parse($date_debut_jour,$date_fin_jour,$i)

{


  global $data_access_squid, $data_access_path, $data_access_squid_sem_dern ;

  if ( $i == 0 ) 
    {

      $fichier = $data_access_path.$data_access_squid ;

    }else{

      $fichier = $data_access_path.$data_access_squid_sem_dern ;

    }


  $liste =  Explose_Fichier ( $fichier ) ;

  foreach ( $liste as $val )  {
	
    // il faut spliter sur plusieurs espaces (pas de explode)
    $vect = split ( '[ ]+' , $val ) ;

    $boolean = IntervalleJour( $vect[0] , $date_debut_jour , $date_fin_jour ) ;

    if ( $boolean == 1 )  {
      
      // transcription "lisible" du timestamp

      $heure = humanDate( $vect[0] ) ;

      $result[] = array(
			'time' => $heure
			,'ip' => $vect[2] 
			,'url' => $vect[6]
			,'referant' => $vect[7]
			) ;

    } else  {
      // rien du tout...
    }
  }

  return $result ;

}


  /*
    ParseRefus
    -
    parse les logs du data-access pour les refusés de SquidGuard
    -
    Paramètres : 
    pas de paramètre
    -
    Résultat : 
    retourne le vecteur avec les données à présenter
    -
    Globales :
    $squidGuard_log_path, $squid_guard_log ;
    - 
  */

function ParseRefus() 
{

  global $data_access_path , $data_access_squid_guard_refus ;
  
  $fichier = $data_access_path.$data_access_squid_guard_refus ;

  $liste = file($fichier) ;

  foreach ($liste as $val)  {
	
    // il faut spliter sur plusieurs espaces (pas de explode)
    $vect = split ('[ ]+',$val) ;


    $result[] = array(
			'date' => $vect[0]
			,'time' => $vect[1]
			,'url' => $vect[4]
			,'ip' => $vect[5] 
			,'referant' => $vect[6]
			) ;

  }

  return $result ;

}

//===============================================================================



  /*
    FormatageResultat
    -
    formatage html (mise dans un tableau) du vecteur résultat du parsing
    -
    Paramètres : 
    $result : vecteur de donnée résulant du parsing
    -
    Résultat : 
    retourne le html formatant les données intéressantes
    -
    Globales :
    aucune
    - 
  */

function FormatageResultat($result)
{

  $buffer = "" ;

  $buffer .= "<BR><TABLE WIDTH=\"770\" ALIGN=\"CENTER\" BORDER=\"0\" CELLPADDING=\"3\" CELLSPACING=\"0\" >" ;
  // première ligne du tableau : description des différents champs 

  $buffer .= "<TR BGCOLOR=\"#DFDFFF\" ALIGN=\"LEFT\">" ;
  $buffer .= "<TD ALIGN=\"LEFT\">date</TD>" ;
  $buffer .= "<TD ALIGN=\"LEFT\">poste client</TD>" ;
  $buffer .= "<TD ALIGN=\"LEFT\">nom de domaine</TD>" ;
  $buffer .= "<TD ALIGN=\"LEFT\">utilisateur</TD>" ;
  $buffer .= "</TR>" ;


  foreach ($result as $val) {

    $time = $val['time'] ;
    $ip = $val['ip'] ;
    $referant = $val['referant'] ;

    // décompostition de l'url
    $url = $val['url'] ;
    $chemin= parse_url($url); 
    $host = $chemin['host'] ;
    $path = $chemin['path'] ;
    // fin décomposition url

    if ( $host == "" )  
      {

	$host = "vide" ;

      }
    
    $buffer .= "<TR BGCOLOR=\"#FFFFCA\" ALIGN=\"left\" VALIGN=\"top\">" ;
    $buffer .= "<TD ALIGN=\"LEFT\">".$time."</TD>" ;
    $buffer .= "<TD ALIGN=\"LEFT\"><A HREF=\"int-affiche-squid-ip.php?url_ip=".$ip."\">".$ip."</A></TD>" ;
    $buffer .= "<TD ALIGN=\"LEFT\"><A HREF=\"int-affiche-squid-host.php?url_host=".$url."\">".$host."</A></TD>" ;
    $buffer .= "<TD ALIGN=\"LEFT\"><A HREF=\"int-affiche-squid-referant.php?url_referant=".$referant."\">".$referant."</A></TD>" ;
    $buffer .= "</TR>" ;

  }

  $buffer .= "</TABLE>\n";

  return $buffer ;

}


  /*
    FormatageResultatRefus
    -
    formatage html (mise dans un tableau) du vecteur résultat du parsing
    sur le refus.log
    -
    Paramètres : 
    vecteur de donnée résulant du parsing
    -
    Résultat :
    retourne le html formatant les données intéressantes
    -
    Globales :
    aucune
    - 
  */
  

function FormatageResultatRefus($result)
{


  $buffer = "" ;

  $buffer .= "<BR><TABLE WIDTH=\"100%\" ALIGN=\"CENTER\" BORDER=\"0\" CELLPADDING=\"3\" CELLSPACING=\"0\" >" ;
  // première ligne du tableau : description des différents champs 

  $buffer .= "<TR BGCOLOR=\"#DFDFFF\" ALIGN=\"LEFT\">" ;
  $buffer .= "<TD ALIGN=\"LEFT\">date</TD>" ;
  $buffer .= "<TD ALIGN=\"LEFT\">heure</TD>" ;
  $buffer .= "<TD ALIGN=\"LEFT\">poste client</TD>" ;
  $buffer .= "<TD ALIGN=\"LEFT\">nom de domaine</TD>" ;
  $buffer .= "<TD ALIGN=\"LEFT\">utilisateur</TD>" ;
  $buffer .= "</TR>" ;


  foreach ($result as $val) {

    $date = $val['date'] ;
    $time = $val['time'] ;
    $ip = $val['ip'] ;
    $referant = $val['referant'] ;

    // décompostition de l'url
    $url = $val['url'] ;
    $chemin= parse_url($url); 
    $host = $chemin['host'] ;
    $path = $chemin['path'] ;
    // fin décomposition url

    
    $buffer .= "<TR BGCOLOR=\"#FFFFCA\" ALIGN=\"left\" VALIGN=\"top\">" ;
    $buffer .= "<TD ALIGN=\"LEFT\">".$date."</TD>" ;
    $buffer .= "<TD ALIGN=\"LEFT\">".$time."</TD>" ;
    $buffer .= "<TD ALIGN=\"LEFT\">".$ip."</TD>" ;
    $buffer .= "<TD ALIGN=\"LEFT\">".$host."</TD>" ;
    $buffer .= "<TD ALIGN=\"LEFT\">".$referant."</TD>" ;
    $buffer .= "</TR>" ;

  }

  $buffer .= "</TABLE>\n";

  return $buffer ;

}


  /*
    FormatageResultatIP
    -
    formatage html (mise dans un tableau) du vecteur résultat du parsing filtré
    -
    Paramètres : 
    $result // vecteur de donnée résulant de la désérialisation
    -
    valeur retournée : 
    retourne le html formatant les données intéressantes
    -
    Globales :
    aucune
    - 
  */


function FormatageResultatIP($result)
{
 
  $buffer = "" ;

  $buffer .= "<BR><TABLE WIDTH=\"770\" ALIGN=\"CENTER\" BORDER=\"0\" CELLPADDING=\"3\" CELLSPACING=\"0\" >" ;
  // première ligne du tableau : description des différents champs 

  $buffer .= "<TR BGCOLOR=\"#DFDFFF\" ALIGN=\"LEFT\">" ;
  $buffer .= "<TD ALIGN=\"LEFT\">date</TD>" ;
  $buffer .= "<TD ALIGN=\"LEFT\">nom de domaine</TD>" ;
  $buffer .= "<TD ALIGN=\"LEFT\">utilisateur</TD>" ;
  $buffer .= "</TR>" ;

  foreach ($result as $val) {


    $time = $val['time'] ;
    $vect_ip = $val['ip'] ;
    $referant = $val['referant'] ;


    // décompostition de l'url
    $url = $val['url'] ;
    $chemin= parse_url($url); 
    $host = $chemin['host'] ;
    $path = $chemin['path'] ;

    if ( $host == "" )  
      {

	$host = "vide" ;

      }

    // fin décomposition url
    // echo "IP select est $ip<br>" ;
     
	$buffer .= "<TR BGCOLOR=\"#FFFFCA\" ALIGN=\"left\" VALIGN=\"top\">" ;
	$buffer .= "<TD ALIGN=\"LEFT\">".$time."</TD>" ;
        $buffer .= "<TD ALIGN=\"LEFT\"><A HREF=\"int-affiche-squid-host.php?url_host=$url\">".$host."</A></TD>" ;
	$buffer .= "<TD ALIGN=\"LEFT\"><A HREF=\"int-affiche-squid-referant.php?url_referant=$referant\">".$referant."</A></TD>" ;
	$buffer .= "</TR>" ;

  }

  $buffer .= "</TABLE>\n";

  return $buffer ;

}



  /*
    FormatageResultatReferant
    -
    formatage html (mise dans un tableau) du vecteur résultat du parsing filtré
    -
    Paramètres : 
    $result // vecteur de donnée résulant de la désérialisation
    $recupreferant // récupération du nom de l'utilisateur en cours
    -
    valeur retournée : 
    retourne le html formatant les données intéressantes
    -
    Globales :
    aucune
    - 
  */

function FormatageResultatReferant($result)
{

  $buffer = "" ;

  $buffer .= "<BR><TABLE WIDTH=\"770\" ALIGN=\"CENTER\" BORDER=\"0\" CELLPADDING=\"3\" CELLSPACING=\"0\" >" ;
  // première ligne du tableau : description des différents champs 

  $buffer .= "<TR BGCOLOR=\"#DFDFFF\" ALIGN=\"LEFT\">" ;
  $buffer .= "<TD ALIGN=\"LEFT\">date</TD>" ;
  $buffer .= "<TD ALIGN=\"LEFT\">poste client</TD>" ;
  $buffer .= "<TD ALIGN=\"LEFT\">nom de domaine</TD>" ;

  $buffer .= "</TR>" ;


  foreach ($result as $val) {

    $time = $val['time'] ;
    $ip = $val['ip'] ;
    $referant = $val['referant'] ;

    // décompostition de l'url
    $url = $val['url'] ;
    $chemin= parse_url($url); 
    $host = $chemin['host'] ;
    $path = $chemin['path'] ;
    if ( $host == "" )  
      {

	$host = "vide" ;

      }

    // fin décomposition url
    
    $buffer .= "<TR BGCOLOR=\"#FFFFCA\" ALIGN=\"left\" VALIGN=\"top\">" ;
    $buffer .= "<TD ALIGN=\"LEFT\">".$time."</TD>" ;
    $buffer .= "<TD ALIGN=\"LEFT\"><A HREF=\"int-affiche-squid-ip.php?url_ip=$ip\">".$ip."</A></TD>" ;
    $buffer .= "<TD ALIGN=\"LEFT\"><A HREF=\"int-affiche-squid-host.php?url_host=$url\">".$host."</A></TD>" ;

    $buffer .= "</TR>" ;

  }

  $buffer .= "</TABLE>\n";

  return $buffer ;

}

/*
FormatageResultatHost
    -
    formatage html (mise dans un tableau) du vecteur résultat du parsing filtré
    -
    Paramètres : 
    $parse_result // vecteur de donnée résulant de la désérialisation
    $base_host // récupération du nom de domaine courant
    -
    Résultat : 
    retourne le html formatant les données intéressantes
    -
    Globales :
    aucune
    - 
 */
function FormatageResultatHost( $parse_result )
{

  $buffer = "" ;

  $buffer .= "<BR><TABLE WIDTH=\"770\" ALIGN=\"CENTER\" BORDER=\"0\" CELLPADDING=\"3\" CELLSPACING=\"0\" >" ;
  // première ligne du tableau : description des différents champs 

  $buffer .= "<TR BGCOLOR=\"#DFDFFF\" ALIGN=\"LEFT\">" ;
  $buffer .= "<TD ALIGN=\"LEFT\">date</TD>" ;
  $buffer .= "<TD ALIGN=\"LEFT\">poste client</TD>" ;
  $buffer .= "<TD ALIGN=\"LEFT\">page visitée</TD>" ;
  $buffer .= "<TD ALIGN=\"LEFT\">utilisateur</TD>" ;
  $buffer .= "</TR>" ;


  foreach ($parse_result as $val) {

    $time = $val['time'] ;
    $ip = $val['ip'] ;
    $referant = $val['referant'] ;

    // décompostition de l'url
    $url = $val['url'] ;
    $chemin= parse_url($url); 
    $host = $chemin['host'] ;
    $path = $chemin['path'] ;
    /*
    if ( $path = "" )  
      {

	$path = "vide" ;

      }
    */
    $int_longueur = strlen( $path ) ;

    // tronquer l'affichage de l'url lorsqu'elle est trop longue
    if ( $int_longueur >= 20 ) 
      {

	$path = substr( $path , 0 , 20 ) ;

      }
    
    // fin décomposition url

    $buffer .= "<TR BGCOLOR=\"#FFFFCA\" ALIGN=\"left\" VALIGN=\"top\">" ;
    $buffer .= "<TD ALIGN=\"LEFT\">".$time."</TD>" ;
    $buffer .= "<TD ALIGN=\"LEFT\"><A HREF=\"int-affiche-squid-ip.php?url_ip=".$ip."\">".$ip."</A></TD>" ;
    $buffer .= "<TD ALIGN=\"LEFT\"><A HREF=\"".$url."\">".$path."</A></TD>" ;
    $buffer .= "<TD ALIGN=\"LEFT\"><A HREF=\"int-affiche-squid-referant.php?url_referant=".$referant."\">".$referant."</A></TD>" ;
    $buffer .= "</TR>" ;

  }

  $buffer .= "</TABLE>\n";

  return $buffer ;

}



/*
=======================================================================

manipulation générale de dates

=======================================================================
*/

  
  /*
    humanDate
    -
    parce qu'on est pas des machine à lire les time stamps...
    il y en a déjà une semblable dans la common lib
    mais c'est pour la date courante et avec un retour chariot
    -
    Paramètres : 
    time_stamp à formater
    -
    Résultat :
    string jours/mois/anneé heure:minute:secondes
    -
    Globales :

    - 
  */

function humanDate( $time_stamp ) 
{
  
  $msg = date( "j/m/Y à H:i:s", $time_stamp ) ;

  return $msg ;

}

//=======================================================================


/*
=======================================================================

 fonctions de sérialisation : 

=======================================================================
*/

/*
Serialize_result
-
sérialisation du vecteur résultat du parsing des logs
pour de le réutiliser dans les pages suivantes
le vecteur est écrit dans un fichier suivant un format maison
-
Paramètres :
$pare_r // le vecteur résultat du parsing des log
-
Résultat :
aucun
-
Globales :
$fichier_serialize 
String2File()
-
 */

function Serialize_result( $parse_r ) 
 
{
  
  global $fichier_serialize ;

  $ligne = "" ;

  foreach ( $parse_r as $clef => $val ) 
 
    {

      // 0: clef 1 : time , 2: ip , 3: url, 4: référant

      $ligne .= $clef."%%".$val['time']."%%".$val['ip']."%%".$val['url']."%%".$val['referant']."\n" ;


    }

  String2File( $ligne , $fichier_serialize ) ;

}


/*
DeSerialize_result
-
désérialisation du vecteur résultat du parsing des logs
pour de le réutiliser dans les pages suivantes
le vecteur est lu depuis un fichier
-
Paramètres :
aucun
-
Résultat :
$parse_r // le vecteur résultat
-
Globales :
$fichier_serialize 
-
 */

function DeSerialize_result() 
 
{

  global $fichier_serialize ;

  $lignes = Explose_Fichier ( $fichier_serialize ) ;

  foreach ( $lignes as $val )
 
    {
      
      $v = split ("%%" ,$val) ;
      $referer = str_replace("\n","",$v[4]) ;
      $buffer = array( 
				      'time' => $v[1]
				      ,'ip' => $v[2]
				      ,'url' => $v[3]
				      ,'referant' => $referer
				      ) ;
      $parse_r [$v[0]] = $buffer ;

    }

  return $parse_r ;

}
//=======================================================================

?>
